import paho.mqtt.client as mqtt
import threading
from loguru import logger
from AbstractMqttHandler import AbstractMqttPublisher
from time import sleep

def mqttCoilSubscriberStart(config, processImage):
    mqttCoilSubscriberThread = MqttCoilSubscriber(config, processImage)
    mqttCoilSubscriberThread.start()


class MqttCoilSubscriber(AbstractMqttPublisher):
    def __init__(self, config, processImage):
        super().__init__(config, processImage)

    def localLoop(self):
        while True:
            sleep(self.config["analogInputPublishPeriod"])

    def onMessage(self, topic, payload):
        logger.warning("mqtt message received: {} -> {}".format(topic, str(payload)))
        coilNum = topic.split('/')[-1]
        try:
            coilNum = int(coilNum)
            if (coilNum < 0) or (coilNum >= self.processImage.numCoils):
                raise ValueError("coilNum too large or negative")

            payload = payload.decode("ascii")
            if payload.lower() in [ "on", "true", "1" ]:
                state = True
            elif payload.lower() in [ "off", "false", "0" ]:
                state = False
            else:
                raise ValueError("Invalid payload")

            with self.processImage:
                self.processImage.setCoil(coilNum, state)
            logger.debug("requested to set coil {} to {}".format(coilNum, state))
        except ValueError as e:
            logger.warning("invavlid topic ({}) or payload ({}) in message, error {}".format(topic, str(payload), e))

    def onConnect(self):
        logger.info("mqtt connected")
        self.client.subscribe("{}/+".format(self.config["digitalOutputTopicPrefix"]))
        logger.info("subscribed to coil topic")
