from threading import Condition, Event
from loguru import logger

def zippingFilter(a, b):
    return [ x for x in enumerate(zip(a, b)) if x[1][0] != x[1][1] ]

class NotInitializedException(Exception): pass

class ProcessImage(Condition):
    def __init__(self):
        super().__init__()
        self.coilEvent = Event()
        self.initialized = False

    def init(self, numCoils, numDiscreteInputs, numAnalogInputs):
        self.numCoils = numCoils
        self.coils = []
        self.shadowCoils = [ None ] * numCoils

        self.numDiscreteInputs = numDiscreteInputs
        self.discreteInputs = []
        self.shadowDiscreteInputs = [ None ] * numDiscreteInputs

        self.numAnalogInputs = numAnalogInputs
        self.analogInputs = []
        self.shadowAnalogInputs = [ None ] * (numAnalogInputs // 8)

        self.initialized = True
    
    def isInitialized(self):
        return self.initialized

    def hasPendingInputChanges(self):
        return (self.discreteInputs != self.shadowDiscreteInputs) or (self.analogInputs != self.shadowAnalogInputs)

    def hasPendingOutputChanges(self):
        return self.shadowCoils != self.coils

    def setDiscreteInputs(self, discreteInputs):
        if not self.initialized:
            raise NotInitializedException
        self.discreteInputs = discreteInputs

    def getChangedDiscreteInputs(self):
        if not self.initialized:
            raise NotInitializedException
        changedDiscreteInputs = zippingFilter(self.discreteInputs, self.shadowDiscreteInputs)
        self.shadowDiscreteInputs = self.discreteInputs[:]
        return changedDiscreteInputs
    
#    def getDiscreteInputs(self):
#        if not self.initialized:
#            raise NotInitializedException
#        return self.discreteInputs

    def setCoils(self, coils):
        if not self.initialized:
            raise NotInitializedException
        self.coils = coils

    def setCoil(self, coilNum, value):
        if not self.initialized:
            raise NotInitializedException
        self.coils[coilNum] = value
        self.coilEvent.set()

#    def getChangedCoils(self):
#        if not self.initialized:
#            raise NotInitializedException
#        changedCoils = zippingFilter(self.coils, self.shadowCoils)
#        self.shadowCoils = self.coils
#        return changedCoils
    
    def getCoils(self):
        if not self.initialized:
            raise NotInitializedException
        self.shadowCoils = self.coils[:]
        return self.coils

    def setAnalogsInputs(self, analogInputs):
        if not self.initialized:
            raise NotInitializedException
        self.analogInputs = analogInputs

    def getChangedAnalogsInputs(self):
        if not self.initialized:
            raise NotInitializedException
        changedAnalogInputs = zippingFilter(self.analogInputs, self.shadowAnalogInputs)
        self.shadowAnalogInputs = self.analogInputs[:]
        return changedAnalogInputs
    
    def getAnalogsInputs(self):
        if not self.initialized:
            raise NotInitializedException
        return self.analogInputs


    

    

