from loguru import logger
import argparse
import configparser

from ProcessImage import ProcessImage
from ModbusHandler import modbusStart
from MqttEventPublisher import mqttEventPublisherStart
from MqttPeriodPublisher import mqttPeriodPublisherStart
from MqttCoilSubscriber import mqttCoilSubscriberStart



logger.info("DigitalTwin1 starting")

parser = argparse.ArgumentParser(description="DigitalTwin1")
parser.add_argument('--config', '-f',
                    help='Config file, default is $pwd/config/config.ini',
                    required=False,
                    default='./config/config.ini')
args = parser.parse_args()

config = configparser.ConfigParser()
config.read(args.config)




processImage = ProcessImage()

modbusStart(config, processImage)
logger.info("Modbus handler running")

mqttEventPublisherStart(config, processImage)
logger.info("MQTT event publisher running")

mqttPeriodPublisherStart(config, processImage)
logger.info("MQTT period publisher running")

mqttCoilSubscriberStart(config, processImage)
logger.info("MQTT coil subscriber running")

logger.info("DigitalTwin1 running")

